﻿using Printing;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Printing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using static Printing.PrintForm;

namespace Acop
{
  [ToolboxItem(true)]
  [ToolboxBitmap(typeof(StatusStrip))]
  public partial class AcopStatusBar : StatusStrip
  {
#   pragma warning disable 1591
    private Boolean isDesignMode;
    private AcopSpider spider;
    private Timer timer;
    private Timer printTimer;
    private ToolStripStatusLabel timeLabel;
    private ToolStripStatusLabel hostLabel;
    private ToolStripStatusLabel versionLabel;
    private Button btnPrint;
    private ToolTip tt = new ToolTip();

    private PrintForm pf;
    private PrintInfoDialog pid;

    private string logPrinter;
    private string logHeader;
    private string hostName;

    private Pen grayPen = new Pen(Color.Gray, 0.5f);

    public AcopStatusBar()
    {
      hostName = Environment.MachineName;
      InitializeComponent();
    }

    private void InitializeComponent()
    {
      isDesignMode = (LicenseManager.UsageMode == LicenseUsageMode.Designtime);
      this.LayoutStyle = ToolStripLayoutStyle.HorizontalStackWithOverflow;
      this.SizingGrip = false;

      timer = new Timer();
      timer.Interval = 1000;
      if(!isDesignMode) timer.Enabled = true;
      timer.Tick += new EventHandler(this.Timer_Tick);

      printTimer = new Timer();
      printTimer.Interval = 500;
      printTimer.Tick += new EventHandler(this.PrintTimer_Tick);

      timeLabel = new ToolStripStatusLabel();
      timeLabel.TextAlign = ContentAlignment.MiddleCenter;
      timeLabel.Alignment = ToolStripItemAlignment.Right;
      timeLabel.Text = DateTime.Now.ToString("dd-MMM-yyyy HH:mm:ss");

      hostLabel = new ToolStripStatusLabel();
      hostLabel.Text = hostName;
      hostLabel.TextAlign = ContentAlignment.MiddleCenter;

      versionLabel = new ToolStripStatusLabel();
      versionLabel.Text = "Acop.net " + AcopUserComponent.acopVersion;
      versionLabel.TextAlign = ContentAlignment.MiddleCenter;

      spider = new AcopSpider();
      spider.Size = new Size(24, 24);
      spider.MinimumSize = spider.Size;

      btnPrint = new Button();
      btnPrint.Text = "Print";
      btnPrint.Font = new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular);
      btnPrint.Size = new Size(30, 20);
      btnPrint.MinimumSize = btnPrint.Size;
      btnPrint.FlatStyle = FlatStyle.Flat;
      btnPrint.FlatAppearance.BorderSize = 0;
      btnPrint.Click += new EventHandler(this.Print_Click);
      tt.SetToolTip(btnPrint, "Print screenshot to logbook");

      this.Paint += new PaintEventHandler(Painting);

      this.Items.Add(new ToolStripControlHost(spider));
      this.Items.Add(new ToolStripSeparator());
      this.Items.Add(new ToolStripControlHost(btnPrint));
      this.Items.Add(new ToolStripSeparator());
      this.Items.Add(hostLabel);
      this.Items.Add(new ToolStripSeparator());
      this.Items.Add(versionLabel);
      this.Items.Add(new ToolStripSeparator());
      this.Items.Add(timeLabel);
    }

    private void Painting(object sender, PaintEventArgs e)
    {
      Graphics g = e.Graphics;
      g.DrawLine(grayPen, 0, 0, this.Width - 1, 0);
    }

    private void Timer_Tick(object sender, EventArgs e)
    {
      timeLabel.Text = DateTime.Now.ToString("dd-MMM-yyyy HH:mm:ss");
    }

    private void PrintTimer_Tick(object sender, EventArgs e)
    {
      printTimer.Enabled = false;
      PrintScreen();
    }

    private void PrintScreen()
    {
      var me = this.FindForm();
      try
      {
        pf = new PrintForm(ref me);
        if (logPrinter == "PrintPreview")
        {
          pf.PrintPreview(20, 40, 40, 40, true, false, true, PrintMode_ENUM.FitToPage, null);
        }
        else
        {
          pf.PrintToLogBook(logPrinter, hostName, LogbookSeverity_ENUM.Info, logHeader, "", true, false, true);
        }
      }
      catch(Exception e)
      {
        Console.WriteLine(e.ToString());
        printTimer.Enabled = false;
      }
    }

    private void Print_Click(object sender, EventArgs e)
    {
      pid = new PrintInfoDialog(this);
      pid.StartPosition = FormStartPosition.CenterParent;
      DialogResult dr = pid.ShowDialog();
      if (dr == DialogResult.OK)
      {
        if (logPrinter == null || logPrinter == "") return;
        printTimer.Enabled = true;
      }
      else
      {
        return;
      }
    }

    private void SetLogPrinter(string value)
    {
      logPrinter = value;
      if (value == null || value == "") btnPrint.Enabled = false;
      else btnPrint.Enabled = true;
    }
    private void setLogHeader(string value)
    {
      logHeader = value;
    }

    public static string[] GetPrintersList()
    {
      int printCount = PrinterSettings.InstalledPrinters.Count;
      ArrayList printers = new ArrayList();

      for(int i = 0; i < printCount; i++)
      {
        String p = PrinterSettings.InstalledPrinters[i];
        if (p.Contains("log")) printers.Add(p);
      }
      printers.Add("PrintPreview");

      return (string[])printers.ToArray(typeof(string));
    }

    #region Properties
    [Category("Acop.Display")]
    [Description("Set the Logbook printer")]
    [TypeConverter(typeof(PrinterConverter))]
    public string Logbook { get { return logPrinter; } set { SetLogPrinter(value); } }
    [Category("Acop.Display")]
    [Description("Set the header message that will be showing in the logbook")]
    public string Header { get { return logHeader; } set { setLogHeader(value); } }
    #endregion
  }

  public class PrinterConverter : StringConverter
  {
#pragma warning disable 1591
    public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
    { //true means show a combobox
      return true;
    }
    public override bool GetStandardValuesExclusive(ITypeDescriptorContext context)
    { //true will limit to list. false will show the list, but allow free-form entry
      return false;
    }
    public override StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
    {
      return new StandardValuesCollection(AcopStatusBar.GetPrintersList());
    }
  }

  public class PrintInfoDialog : Form
  {
#   pragma warning disable 1591
    AcopStatusBar bar;
    public PrintInfoDialog(AcopStatusBar bar)
    {
      this.bar = bar;
      InitializeComponent();
    }

    private void btn_OK_Click(object sender, EventArgs e)
    {
      bar.Logbook = cb_Printers.Text;
      this.Dispose();
    }

    private void btn_Cancel_Click(object sender, EventArgs e)
    {
      this.Dispose();
    }

    private void InitializeComponent()
    {
      this.btn_OK = new Button();
      this.btn_Cancel = new Button();
      this.cb_Printers = new ComboBox();
      this.label1 = new Label();
      this.SuspendLayout();
      // 
      // btn_OK
      // 
      this.btn_OK.DialogResult = DialogResult.OK;
      this.btn_OK.Location = new Point(55, 112);
      this.btn_OK.Name = "btn_OK";
      this.btn_OK.Size = new Size(75, 23);
      this.btn_OK.TabIndex = 0;
      this.btn_OK.Text = "OK";
      this.btn_OK.UseVisualStyleBackColor = true;
      this.btn_OK.Click += new EventHandler(this.btn_OK_Click);
      // 
      // btn_Cancel
      // 
      this.btn_Cancel.DialogResult = DialogResult.Cancel;
      this.btn_Cancel.Location = new Point(188, 112);
      this.btn_Cancel.Name = "btn_Cancel";
      this.btn_Cancel.Size = new Size(75, 23);
      this.btn_Cancel.TabIndex = 1;
      this.btn_Cancel.Text = "Cancel";
      this.btn_Cancel.UseVisualStyleBackColor = true;
      this.btn_Cancel.Click += new EventHandler(this.btn_Cancel_Click);
      // 
      // cb_Printers
      // 
      this.cb_Printers.FormattingEnabled = true;
      this.cb_Printers.Location = new Point(55, 56);
      this.cb_Printers.Name = "cb_Printers";
      this.cb_Printers.Size = new Size(208, 21);
      this.cb_Printers.TabIndex = 2;
      this.cb_Printers.Items.AddRange(AcopStatusBar.GetPrintersList());
      this.cb_Printers.SelectedText = bar.Logbook;
      // 
      // label1
      // 
      this.label1.AutoSize = true;
      this.label1.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
      this.label1.Location = new System.Drawing.Point(90, 19);
      this.label1.Name = "label1";
      this.label1.Size = new System.Drawing.Size(146, 20);
      this.label1.TabIndex = 3;
      this.label1.Text = "Print screenshot to:";
      // 
      // Form3
      // 
      this.AcceptButton = this.btn_OK;
      this.AutoScaleDimensions = new SizeF(6F, 13F);
      this.AutoScaleMode = AutoScaleMode.Font;
      this.CancelButton = this.btn_Cancel;
      this.ClientSize = new Size(328, 147);
      this.ControlBox = false;
      this.Controls.Add(this.label1);
      this.Controls.Add(this.cb_Printers);
      this.Controls.Add(this.btn_Cancel);
      this.Controls.Add(this.btn_OK);
      this.Name = "Form3";
      this.Text = "Print to logbook";
      this.ResumeLayout(false);
      this.PerformLayout();

    }

    private Button btn_OK;
    private Button btn_Cancel;
    private ComboBox cb_Printers;
    private Label label1;
  }
}
